package traffic;

/**
 * 交通灯
 * 	每个路口3盏灯，共12盏，其中南北方向可共用左转灯和直行灯，东西方向可共用左转灯和直行灯。所有路口可共用一盏右转灯，简化后共5盏灯
 * 	每盏灯有红绿两种状态，每种状态能够切换为另一种状态。右转灯保持绿
 * 		注意切换时的逻辑，当前灯不但要改变自己的状态，还要改变下一个灯的状态
 *	
 * @author lihao
 *
 */
public enum TrafficLamp {
	// 南北直行灯、南北左转灯、东西直行灯、东西左转灯、右转灯
	SNSTRAIGHT(false, "SNLEFT"), SNLEFT(false, "EWSTRAIGHT"), 
	EWSTRAIGHT(false, "EWLEFT"), EWLEFT(false, "SNSTRAIGHT"), 
	RIGHT(true, null);
	
	private volatile boolean passable; // 当前灯是否可通行
	private String next; // 下一盏灯. 由于初始化顺序，无法直接获取未初始化的TrafficLamp对象，因而暂时用string
	
	private TrafficLamp(boolean passable, String next) {
		this.passable = passable;
		this.next = next;
	}
	
	public boolean isPassable() {
		return passable;
	}
	
	/**
	 * 切换为绿灯
	 */
	public void setPassable() {
		this.passable = true;
		System.out.println(this + " turn green");
	}

	/**
	 * 切换为红灯，同时下一盏灯切换为绿灯
	 */
	public TrafficLamp setImpassable() {
		if (this == TrafficLamp.RIGHT) // 右转灯永不切换
			return this;
		this.passable = false;
		System.out.println(this + " turn red");
		TrafficLamp nextLamp = TrafficLamp.valueOf(this.next);
		nextLamp.setPassable();
		return nextLamp;
	}
}
