package traffic;
import java.util.concurrent.*;

/**
 * 交通灯控制器
 * 	每隔一定时间进行交通灯切换
 * @author lihao
 *
 */
public class TrafficLampControl {
	/** 交通灯切换时间间隔(单位：秒) */
	private final int interval; 
	
	/** 执行切换任务的定时器 */
	private ScheduledThreadPoolExecutor exec = new ScheduledThreadPoolExecutor(1);
	
	/** 当前可通行的交通灯 */
	private TrafficLamp currentLamp;
	
	public TrafficLampControl(TrafficLamp initPassableLamp, int interval) {
		this.currentLamp = initPassableLamp;
		currentLamp.setPassable(); // 初始可通过的灯
		this.interval = interval;
		// 定时切换交通灯
		exec.scheduleAtFixedRate(new Runnable() {
			public void run() {
				currentLamp = currentLamp.setImpassable();
			}
		}, interval, interval, TimeUnit.SECONDS);
	}

	/** 关闭交通灯切换线程 */
	public void shutdown() {
		exec.shutdownNow();
	}

}